/* 
 * File:   SPI_EEPROM.c
 *
 * P?ipojen SPI_EEPROM(25LC640,25LC512,25LC1024)
 * k PIC18F26K20,extern krystal 16MHz
 *
 * Author: Standa
 *
 * Created on 10. leden 2013, 6:52
 */

#include <p18cxxx.h>
#include "configure.h"
#include <delays.h>
#include "LCD_4bit.h"

#define WRSR 0x01
#define WRITE 0x02
#define READ 0x03
#define WRDI 0x04
#define RDSR 0x05
#define WREN 0x06

#define EEPROM_25LC1024 0
#define EEPROM_25LC640 1
#define EEPROM_25LC512 2

const char rom *text1_wr = "25LC512";
const char rom *text2_wr = "25LC640";
const char rom *text3_wr = "25LC1024";

char ram text1_rd[8];
char ram text2_rd[8];
char ram text3_rd[9];

void _delay3s(void)
{
    unsigned char i;
    for(i = 1;i<7;i++)          //6*500ms = 3s
    Delay10KTCYx(200);          //10000x200x0,25 = 500ms
}

void SPI_Init(void)
{
    TRISC = 0b11010000;         //PC3(SCK),PC5(SDO),PC4(SDI),PC0..PC2(CSx)
    LATC = 0b11111111;
    SSPSTAT = 0b11000000;       //SMP = 1,CKE = 1
    SSPCON1 = 0b00100000;       //SPI on, Fcpu/4 (16MHz/4/4 = 1MHz)
}

unsigned char SPI_Transfer(unsigned char data)
{
    unsigned char byte;
    SSPBUF = data;
    while (!(SSPSTATbits.BF));
    byte = SSPBUF;
    return byte;
}

void SPI_Wait(unsigned char select)
{
    unsigned char status = 0;
    do
    {
        LATC &= ~(1<<select);
        SPI_Transfer(RDSR);
        status = SPI_Transfer(0);
        LATC |= (1<<select);
    }
    while(status&0x01);
}

void Write_EEPROM_1k_512k(unsigned int adrress,unsigned char data,unsigned char select)
{
    SPI_Wait(select);
    LATC &= ~(1<<select);
    SPI_Transfer(WREN);
    LATC |= (1<<select);
    LATC &= ~(1<<select);
    SPI_Transfer(WRITE);
    SPI_Transfer((adrress&0xFF00) >> 8);
    SPI_Transfer(adrress&0x00FF);
    SPI_Transfer(data);
    LATC |= (1<<select);
}

unsigned char Read_EEPROM_1k_512k(unsigned int adrress,unsigned char select)
{
    unsigned char data = 0;
    SPI_Wait(select);
    LATC &= ~(1<<select);
    SPI_Transfer(READ);
    SPI_Transfer((adrress&0xFF00) >> 8);
    SPI_Transfer(adrress&0x00FF);
    data = SPI_Transfer(0);
    LATC |= (1<<select);
    return data;
}
void Write_EEPROM_1024k(unsigned short adrress,unsigned char data,unsigned char select)
{
    SPI_Wait(select);
    LATC &= ~(1<<select);
    SPI_Transfer(WREN);
    LATC |= (1<<select);
    LATC &= ~(1<<select);
    SPI_Transfer(WRITE);
    SPI_Transfer((adrress&0x00FF0000) >> 16);
    SPI_Transfer((adrress&0x0000FF00) >> 8);
    SPI_Transfer(adrress&0x000000FF);
    SPI_Transfer(data);
    LATC |= (1<<select);
}

unsigned char Read_EEPROM_1024k(unsigned short adrress,unsigned char select)
{
    unsigned char data = 0;
    SPI_Wait(select);
    LATC &= ~(1<<select);
    SPI_Transfer(READ);
    SPI_Transfer((adrress&0x00FF0000) >> 16);
    SPI_Transfer((adrress&0x0000FF00) >> 8);
    SPI_Transfer(adrress&0x000000FF);
    data = SPI_Transfer(0);
    LATC |= (1<<select);
    return data;
}
/***********************************************************************
 *
 *      Hlavn program
 *
 ***********************************************************************/
void main(void)
{
    unsigned char x;

    SPI_Init();
    Lcd_Init();
//-----------------------------------------------------------------------
    Lcd_Out_ROM("Write: ");
    Lcd_Out_ROM(text1_wr);
    Lcd_Gotoxy(2,0);
    Lcd_Out_ROM("Read: ");

    x = 0;
    while(text1_wr[x]!= 0)
    {
        Write_EEPROM_1k_512k(x,text1_wr[x],EEPROM_25LC512);
        x++;
    }

    for(x=0;x<7;x++)
    {
        text1_rd[x] = Read_EEPROM_1k_512k(x,EEPROM_25LC512);
    }

    Lcd_Out_RAM(text1_rd);

    _delay3s();

    Lcd_Cmd(LCD_CLR);
    Lcd_Cmd(LCD_HOME);
//---------------------------------------------------------------------------
    Lcd_Out_ROM("Write: ");
    Lcd_Out_ROM(text2_wr);
    Lcd_Gotoxy(2,0);
    Lcd_Out_ROM("Read: ");

    x = 0;
    while(text2_wr[x]!= 0)
    {
        Write_EEPROM_1k_512k(x,text2_wr[x],EEPROM_25LC640);
        x++;
    }

    for(x=0;x<7;x++)
    {
        text2_rd[x] = Read_EEPROM_1k_512k(x,EEPROM_25LC640);
    }

    Lcd_Out_RAM(text2_rd);

    _delay3s();

    Lcd_Cmd(LCD_CLR);
    Lcd_Cmd(LCD_HOME);
//-----------------------------------------------------------------------------
    Lcd_Out_ROM("Write: ");
    Lcd_Out_ROM(text3_wr);
    Lcd_Gotoxy(2,0);
    Lcd_Out_ROM("Read: ");

    x = 0;
    while(text3_wr[x]!= 0)
    {
        Write_EEPROM_1024k(x,text3_wr[x],EEPROM_25LC1024);
        x++;
    }

    for(x=0;x<8;x++)
    {
        text3_rd[x] = Read_EEPROM_1024k(x,EEPROM_25LC1024);
    }

    Lcd_Out_RAM(text3_rd);
//---------------------------------------------------------------------------
    while(1)
    {
    }
}
/*****************************************************************************/
